#include "nield.h"

static FILE *dbg_fd;

/*
 * open a debug file
 */
int open_dbg(char *filename)
{
	dbg_fd = fopen(filename, "a");
	if(dbg_fd == NULL) {
		fprintf(stderr, "[ERROR] %s: fopen(): %s\n", __func__, strerror(errno));
		fprintf(stderr, "[ERROR] %s: can't open debug file(%s)\n", __func__, filename);
		return(-1);
	}

	return(0);
}

/*
 * record debug messages
 */
void rec_dbg(char *format, ...)
{
	va_list ap;
	struct timeval tv;
	struct tm *tm;
	char buf[512] = "";

	va_start(ap, format);

	vsnprintf(buf, sizeof(buf), format, ap);

	gettimeofday(&tv, NULL);
	tm = localtime(&tv.tv_sec);

	fprintf(dbg_fd, "[%04d-%02d-%02d %02d:%02d:%02d.%06ld] %s\n",
		tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
		tm->tm_hour, tm->tm_min, tm->tm_sec, tv.tv_usec, buf);
	fflush(dbg_fd);

	va_end(ap);
}

/*
 * close a debug file
 */
void close_dbg(void)
{
	if(dbg_fd)
		fclose(dbg_fd);
}
