#ifndef _NIELD_
#define _NIELD_

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include <netinet/in.h>

#include <sys/types.h>
#include <sys/socket.h>

#include <asm/types.h>
#include <linux/rtnetlink.h>
#include <linux/if_arp.h>
#include <linux/fib_rules.h>

#include <linux/if.h> /* IFNAMSIZ */
#include <net/ethernet.h> /* ETHER_ADDR_LEN */

#include <linux/ip.h> /* struct iphdr */
#include <linux/if_vlan.h> /* VLAN_FLAG_* */
#include <linux/if_tunnel.h> /* IFLA_GRE_* */

#include <sys/time.h>
#include <time.h>

#include "list_nield.h"
#include "af_type.h"

/* default value */
#define NIELD_VERSION      "0.22"
#define NIELD_USAGE        "[-vh46inar] [-l log_file] [-d debug_file]"
#define LOG_FILE_DEFAULT   "/var/log/nield.log"
#define DEBUG_FILE_DEFAULT "/var/log/nield.dbg"
#define LOCK_FILE          "/var/run/nield.pid"
#define MAX_STR_SIZE       128
#define MAX_MSG_SIZE       256
#define MODULE_NAME_LEN    (64 - sizeof(unsigned long))

/* option flag */
#define F_DEBUG     0x0001
#define F_IPV4      0x0002
#define F_IPV6      0x0004
#define F_LINK      0x0008
#define F_NEIGH     0x0010
#define F_IFADDR    0x0020
#define F_ROUTE     0x0040
#define F_RULE      0x0080

/* interface message type flag */
#define IF_ADD    0x0001
#define IF_DEL    0x0002
#define IF_CHANGE 0x0004

/* interface list entry format */
struct iflist_entry {
	unsigned index;
	char name[IFNAMSIZ];
	unsigned char addr[INET6_ADDRSTRLEN+1];
	unsigned char brd[INET6_ADDRSTRLEN+1];
	unsigned int flags;
	unsigned short type;
	unsigned short vid;
	unsigned short mtu;
	char kind[MODULE_NAME_LEN];
	unsigned int master;
	unsigned short br_attached;
	struct list_head list;
};

/* neighbor discovery cache format */
struct ndlist_entry {
	unsigned ifindex;
	char ifname[IFNAMSIZ];
	char ipaddr[INET6_ADDRSTRLEN+1];
	unsigned char lladdr[ETHER_ADDR_LEN];
	struct list_head list;
};

/* nield.c */
void close_exit(int sock, int log_flag);
int set_options(int argc, char *argv[]);
int open_lock(void);
void write_lock(void);
void close_lock(void);
int init_daemon(void);
void sigterm_handler(int sig);
void sigint_handler(int sig);
int set_signal_handlers(void);
int open_netlink_socket(unsigned groups);
int send_request(int sock, int type, int family);
int recv_reply(int sock, int type);
void recv_events(int sock);
int parse_events(struct msghdr *mhdr);

/* nlmsg.c */
void debug_nlmsg(struct nlmsghdr *nlh);
const char *debug_n2s_nlmsg_type(int type);

/* ifimsg.c */
int create_iflist(struct msghdr *msg);
void print_iflist(void);
char *if_indextoname_from_iflist(unsigned index, char *ifname);
char *if_indextoname_from_ifhist(unsigned index, char *ifname);
unsigned short get_type_from_iflist(unsigned index);
unsigned short get_type_from_ifhist(unsigned index);
int parse_ifimsg(struct nlmsghdr *nlh);
void inet_ntop_if_addr(unsigned short type, unsigned char *addr, int addrlen, char *addrstr, int addrstrlen);
void debug_ifimsg(struct ifinfomsg *ifim, struct rtattr *ifla[], int ifim_len);
void debug_vlaninfo(struct rtattr *vlaninfo[]);
void debug_vlanqos(struct rtattr *vlanqos[]);
void debug_greinfo(struct rtattr *greinfo[]);
const char *debug_n2s_ifi_type(int type);
void debug_n2s_ifi_flags(int flags, char *flags_list);
const char *debug_n2s_if_oper_state(int state);
const char *debug_n2s_ifla_linkmode(int mode);
void debug_n2s_vlan_flags(int flags, char *flags_list);
void debug_n2s_gre_flags(int flags, char *flags_list);

/* ifamsg.c */
int parse_ifamsg(struct nlmsghdr *nlh);
void debug_ifamsg(struct ifaddrmsg *ifam, struct rtattr *ifa[], int ifam_len);
void debug_n2s_ifa_flags(int flags, char *flags_list);
const char *debug_n2s_ifa_scope(int scope);

/* ndmsg.c */
int create_ndlist(struct msghdr *msg);
void print_ndlist(void);
int parse_ndmsg(struct nlmsghdr *nlh);
void debug_ndmsg(struct ndmsg *ndm, struct rtattr *nda[], int ndm_len);
const char *debug_n2s_ndm_state(int state);
void debug_n2s_ndm_flags(int flags, char *flags_list);
const char *debug_n2s_ndm_type(int type);

/* rtmsg.c */
int parse_rtmsg(struct nlmsghdr *nlh);
void parse_n2s_rtm_type(unsigned char ntype, char *stype);
void parse_n2s_rtm_protocol(int nproto, char *sproto);
void parse_n2s_rtm_table(unsigned ntable, char *stable);
void debug_rtmsg(struct rtmsg *rtm, struct rtattr *rta[], int rtm_len);
const char *debug_n2s_rtm_table(int table);
const char *debug_n2s_rtm_protocol(int protocol);
const char *debug_n2s_rtm_scope(int scope);
const char *debug_n2s_rtm_type(int type);
void debug_n2s_rtm_flags(int flags, char *flags_list);
void debug_n2s_rtnh_flags(int flags, char *flags_list);

/* frhdr.c */
int parse_frhdr(struct nlmsghdr *nlh);
void parse_n2s_frh_action(unsigned char nact, char *sact);
void debug_frhdr(struct fib_rule_hdr *frh, struct rtattr *fra[], int frh_len);
const char *debug_n2s_frh_action(int action);
void debug_n2s_frh_flags(int flags, char *flags_list);

/* log.c */
int open_log(char *filename);
void rec_log(char *format, ...);
void close_log(void);

/* debug.c */
int open_dbg(char *filename);
void rec_dbg(char *format, ...);
void close_dbg(void);

/* af_type.c */
const char *debug_n2s_af_type(int type);

#endif /* _NIELD_ */
