#ifndef _RTNETLINK_
#define _RTNETLINK_

/*
 * parse route attributes
 */
static inline void parse_rtattr(struct rtattr *tb[], int max, struct rtattr *rta, int len)
{
	memset(tb, 0, sizeof(struct rtattr *) * (max + 1));
	while (RTA_OK(rta, len)) {
		if (rta->rta_type <= max)
			tb[rta->rta_type] = rta;
		rta = RTA_NEXT(rta, len);
	}
}

/*
 * parse route attributes nested
 */
static inline void parse_nested_rtattr(struct rtattr *tb[], int max, struct rtattr *rta)
{
	parse_rtattr(tb, max, RTA_DATA(rta), RTA_PAYLOAD(rta));
}

#ifndef IFLA_RTA
#define IFLA_RTA(r) \
	((struct rtattr*)(((char*)(r)) + NLMSG_ALIGN(sizeof(struct ifinfomsg))))
#endif

#ifndef IFLA_PAYLOAD
#define IFLA_PAYLOAD(n)	NLMSG_PAYLOAD(n,sizeof(struct ifinfomsg))
#endif

/*
 * parse interface infromation attributes
 */
static inline void parse_ifinfo(struct rtattr *tb[], struct nlmsghdr *nlh)
{
	struct ifinfomsg *ifim = NLMSG_DATA(nlh);
	parse_rtattr(tb, IFLA_MAX, IFLA_RTA(ifim), IFLA_PAYLOAD(nlh));
}

/*
 * parse link information attributes
 */
static inline void parse_linkinfo(struct rtattr *tb[], struct rtattr *linkinfo)
{
	parse_nested_rtattr(tb, IFLA_INFO_MAX, linkinfo);
}

/*
 * parse vlan information attributes
 */
static inline void parse_vlaninfo(struct rtattr *tb[], struct rtattr *vlaninfo)
{
	parse_nested_rtattr(tb, IFLA_VLAN_MAX, vlaninfo);
}

/*
 * parse vlan qos information attributes
 */
static inline void parse_vlanqos(struct rtattr *tb[], struct rtattr *vlanqos)
{
	parse_nested_rtattr(tb, IFLA_VLAN_QOS_MAX, vlanqos);
}

/*
 * parse gre information attributes
 */
static inline void parse_greinfo(struct rtattr *tb[], struct rtattr *greinfo)
{
	parse_nested_rtattr(tb, IFLA_GRE_MAX, greinfo);
}

#ifndef NDA_RTA
#define NDA_RTA(r) \
	((struct rtattr*)(((char*)(r)) + NLMSG_ALIGN(sizeof(struct ndmsg))))
#endif

#ifndef NDA_PAYLOAD
#define NDA_PAYLOAD(n)	NLMSG_PAYLOAD(n,sizeof(struct ndmsg))
#endif

/*
 * parse neighbor discovery attributes
 */
static inline void parse_ndisc(struct rtattr *tb[], struct nlmsghdr *nlh)
{
	struct ndmsg *ndm = NLMSG_DATA(nlh);
	parse_rtattr(tb, NDA_MAX, NDA_RTA(ndm), NDA_PAYLOAD(nlh));
}

#ifndef IFA_RTA
#define IFA_RTA(r) \
	((struct rtattr*)(((char*)(r)) + NLMSG_ALIGN(sizeof(struct ifaddrmsg))))
#endif

#ifndef IFA_PAYLOAD
#define IFA_PAYLOAD(n)	NLMSG_PAYLOAD(n,sizeof(struct ifaddrmsg))
#endif

/*
 * parse interface address attributes
 */
static inline void parse_ifaddr(struct rtattr *tb[], struct nlmsghdr *nlh)
{
	struct ifaddrmsg *ifam = NLMSG_DATA(nlh);
	parse_rtattr(tb, IFA_MAX, IFA_RTA(ifam), IFA_PAYLOAD(nlh));
}

#ifndef FRA_RTA
#define FRA_RTA(r) \
	((struct rtattr *)(((char*)(r)) + NLMSG_ALIGN(sizeof(struct fib_rule_hdr))))
#endif

#ifndef FRA_PAYLOAD
#define FRA_PAYLOAD(n) NLMSG_PAYLOAD(n,sizeof(struct fib_rule_hdr))
#endif

/*
 * parse fib rule attributes
 */
static inline void parse_frule(struct rtattr *tb[], struct nlmsghdr *nlh)
{
	struct fib_rule_hdr *frh = NLMSG_DATA(nlh);
	parse_rtattr(tb, FRA_MAX, FRA_RTA(frh), FRA_PAYLOAD(nlh));
}

#endif /* _RTNETLINK_ */
